/*
 KEY MIDI
 Arduino Nano 5V / 16Mz
 Avec Afficheur en option SSD1306 128x64 sur I2C
 Led verte et rouge
 10 'Touchpad' avec resistances de 1M à zero
 Liaison Midi sur TX RX + Liaison serie sur ces même TX RX
 
 Version Finale du 01 12 2016
 
    1 2 3      0 Court => changement de bank et double clik sur led rouge
  0 4 5 6      0 Long  => Passe en mode program led verte ON ... Si 0 pas d'enregistrement, si 1-9 fixe en EEPROM
    7 8 9      1 & 7 => Format si suivi d'un appui sur 0 dans les 3 secondes
               2 & 8 => Affichage de la RAM libre (si afficheur) et repasse en mode MIDI
               3 & 9 => Passe en mode serie 9600 Bauds et affiche les programmes enregistrés
               3 & 7 => Force le defrag de l'EEPROM et (si afficheur) affiche la place EEPROM libre
               1 & 9 => si afficheur affiche les timmings Touch
               4 & 6 => Extinction si fonctionnement sur pile.
 */
#include <EEPROM.h>
#include <MemoryFree.h>
#include <avr/pgmspace.h>

#include <TimerOne.h> // Pour la gestion des leds

#include <SPI.h>
#include <Wire.h>

#include <Adafruit_GFX.h>              // OPTION
#include <Adafruit_SSD1306.h>          //
#define OLED_RESET 4                    //
Adafruit_SSD1306 display(OLED_RESET);  // Jaune sur SCL et Vert sur SDA

const byte ledRg = 17; 
const byte ledVt = 16; 

const byte TouchPin[10] = {2,3,4,5,6,7,8,9,15,14}; // ATTENTION LE BOUTON MOD est le 10ieme donc le numéro 9
//     15  5  7   Disposition du clavier et Pins     1 2 3
//  14  2  4  9                        0 = MOD ==> 0 4 5 6
//      3  6  8                                      7 8 9
const byte TouchCod[10] = {4,7,5,2,8,3,9,6,1,0}; // La position sur un clavier standard 0 123 456 589
static byte TouchTim[10] ;
static byte TouchMod[10] ;
byte TouchBas[10] = {3,2,3,2,4,3,3,2,3,6}; // Temps de bases autocalibrés
const byte Seuil = 2; // Sensibilité
static byte NTouch =0; 

static volatile unsigned int ComptAutoOff = 0; // Augmente de 20 par seconde
const unsigned int ComptAutoOffMAX = 36000; // 36000 # 30 mins
const byte AutoOffPin = 12;

static volatile unsigned int LedCompt = 0;

static byte note,midiByte;

static byte FatAdress,Fat;
// 1 Byte config + 9 (pads) x 2 (banks) x (2 bytes position + 1 byte longueur) donc longueur < 256 bytes
                          // Byte config : 10100000 + ??? 
static int AdressEEprom, AdressEEpromMax, AdressEEpromMin;
static byte TaillEEprom;

static volatile byte MOD = 0;
static volatile byte OldMOD = 0;

static int CmptMidi = 0;
const byte BufferMidiMax = 192;
static byte BufferMidi [BufferMidiMax];

static byte i,iMax,Pad ,j, jMax, NoDisplay ;

//**********************************************************************************
void GestionLeds(void){
  LedCompt++; ComptAutoOff++;
  if (MOD == 0) { // if or switch case ...
     if ((LedCompt & B11111) == B11111) {digitalWrite(ledRg, HIGH);} else {digitalWrite(ledRg, LOW);}     
  }
  if (MOD == 1) {
     if ((LedCompt & B11111) == B11111 || (LedCompt & B11111) == B11000 ) {digitalWrite(ledRg, HIGH);} else {digitalWrite(ledRg, LOW);}
  }
  if (MOD == 2) {
     digitalWrite(ledVt, HIGH);
     if ((LedCompt & B11) == B11) {digitalWrite(ledRg, HIGH);} else {digitalWrite(ledRg, LOW);}
  }
  if (MOD == 3) {
     if ((LedCompt & B11) == B10) {digitalWrite(ledVt, HIGH);} else {digitalWrite(ledVt, LOW);}
     if ((LedCompt & B11) == B11) {digitalWrite(ledRg, HIGH);} else {digitalWrite(ledRg, LOW);}
  }
  if (MOD == 6) {
     if (digitalRead(ledRg) == digitalRead(ledVt)) {digitalWrite(ledVt, !digitalRead(ledVt));}    
     if ((LedCompt & B111) == B111) {digitalWrite(ledVt,!digitalRead(ledVt) );digitalWrite(ledRg,!digitalRead(ledRg) );}
  }
  if (MOD == 7) {
     if (digitalRead(ledRg) != digitalRead(ledVt)) {digitalWrite(ledVt, !digitalRead(ledVt));}    
     if ((LedCompt & B111) == B111) {digitalWrite(ledVt,!digitalRead(ledVt) );digitalWrite(ledRg,!digitalRead(ledRg) );}
  }
  if (MOD == 9) {
     if (digitalRead(ledRg) != digitalRead(ledVt)) {digitalWrite(ledVt, !digitalRead(ledVt));}    
     if ((LedCompt & B11) == B11) {digitalWrite(ledVt,!digitalRead(ledVt) );digitalWrite(ledRg,!digitalRead(ledRg) );}
  }
  
}
//**********************************************************************************
// SETUP
void setup() {
  pinMode(ledRg, OUTPUT);  pinMode(ledVt, OUTPUT);
  
  digitalWrite(ledVt, HIGH); // USER PRESS ON
  
  pinMode(AutoOffPin, OUTPUT); digitalWrite(AutoOffPin, HIGH); // AutoON
  
  //Serial.begin (9600); Serial.println(""); Serial.println(F("KEY MIDI ...")); // TESTs
  
  // Test la présence de l'afficheur (Display here ???)
  Wire.begin();
  NoDisplay = 1; delay (500);
  Wire.beginTransmission(0x3C); delay (500);
  if (Wire.endTransmission() == 0) {NoDisplay = 0;}
    
  if (!NoDisplay) {
    display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3D (for the 128x64)
    delay (1000);
    display.clearDisplay();   // clears the screen and buffer  
    display.setTextColor(WHITE); 
    display.setTextSize(2); display.setCursor(0,8); display.println(F(" KEY MIDI"));
    display.setTextSize(1); display.setCursor(0,32); display.print(F("         Luc BRUN")); // display.println(0xDEADBEEF, HEX);
    display.setTextSize(1); display.setCursor(0,52); display.print(F("         Dec 2016")); // display.println(0xDEADBEEF, HEX);
    display.display();
    delay(1000);
    display.invertDisplay(true);
    delay(1000); 
    display.invertDisplay(false);
    delay(500); }
  
  Timer1.initialize(50000);
  Timer1.attachInterrupt(GestionLeds); // Run every 0.05 seconds

  
  Serial.end();
  
  //Serial.begin (9600);  Serial.println("Hello Luc"); // Pour les tests
  
  Serial.begin (31250);  // Pour la liaison MIDI
  note = 0x40; MidiNote(0x90, note, 0x45); delay(400); MidiNote(0x90, note, 0x00);
  
  //EEPROM.write(0, 0); // ATTENTION force le formatage de la FAT EEPROM
  
  //Lire Fat
  Fat = EEPROM.read(0);

  if ((Fat & B10100000) != B10100000) {
    EEPROM.write(0, B10100000); // ATTENTION 3,3 ms
    if (!NoDisplay) {
      display.setTextColor(WHITE,BLACK); display.clearDisplay(); 
      display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
      display.setTextSize(2); display.setCursor(0,22);  display.print(F(" FORMAT   ")); 
      display.setTextSize(2); display.setCursor(0,40);  display.print(F("EEPROM ...")); 
      display.display(); display.invertDisplay(true);    delay(5000); display.invertDisplay(false);} 
    for (i=0; i<18;i++) {
      AdressEEprom = 55 + i ; // Les adresses après formatage sont empiler et de 1 byte
      EEPROM.write(1+3*i, AdressEEprom >> 8);// Position byte haut 
      EEPROM.write(2+3*i, AdressEEprom & B11111111);// Position byte bas
      EEPROM.write(3+3*i, 0);// Longueur = 0
    }
  }
  else {
    if (!NoDisplay) {
      display.setTextColor(WHITE,BLACK); display.clearDisplay(); 
      display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
      display.setTextSize(2); display.setCursor(0,24);  display.print(F("EEPROM FAT")); 
      display.setTextSize(2); display.setCursor(0,44);  display.print(F(" IS GOOD !")); 
      display.display();   delay(2000);  }
  }

  if (!NoDisplay) {
    display.setTextColor(WHITE,BLACK); display.clearDisplay(); 
    display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
    display.setTextSize(2); display.setCursor(0,24);  display.print(F(" FREE RAM ")); 
    display.setTextSize(2); display.setCursor(0,44);  display.print(freeMemory() );
 
    display.display(); display.clearDisplay();display.setTextColor(WHITE, BLACK); }
  
  delay(2000);    
   
  digitalWrite(ledVt, LOW);
  
  // Autocalibrage TouchBas[i]
  for (i=0; i<10;i++) {TouchBas[i]=0;}
  for (j=0; j<4;j++) {
    for (i=0; i<10;i++) {
      TouchTim[i] =0;
      pinMode(TouchPin[i], OUTPUT); digitalWrite(TouchPin[i], HIGH); delay (10); pinMode(TouchPin[i], INPUT);
      while (TouchTim[i]++ < 250 && digitalRead(TouchPin[i])) {}
      if (TouchTim[i]>250) {TouchTim[i]=50;} // Cas d'un problème de lecture
      TouchBas[i] = TouchBas[i] + TouchTim[i]; }
    delay (100);  
    }
  for (i=0; i<10;i++) {TouchBas[i]=TouchBas[i] >> 2;} // On divise par 4
   
  MOD = 0;

}
//**********************************************************************************
void loop() {
  GestionTouch();
  if (NTouch != 0) {digitalWrite(ledVt, HIGH);delay (100);}
   
  if (TouchMod[6] && TouchMod[8] && !NoDisplay) { // Affiche les Timing Touch
    TouchMod[6]=0; TouchMod[8]=0; NTouch=0;
    if (MOD == 13) {MOD =0;} else {MOD = 13;}
  }
  
  if (MOD == 13) {
    NTouch =0;
    for (i=0; i<10; i++) {TouchMod[i]=0;}
    if (!NoDisplay) {
      display.setTextColor(WHITE,BLACK); display.clearDisplay();display.setTextSize(2);
      display.setCursor(0,4); 
      display.print(TouchTim[0]);display.print(F(" "));
      display.print(TouchTim[1]);display.print(F(" "));
      display.print(TouchTim[2]);display.print(F(" "));
      display.print(TouchTim[3]);
      display.setCursor(0,24); 
      display.print(TouchTim[4]);display.print(F(" "));
      display.print(TouchTim[5]);display.print(F(" "));
      display.print(TouchTim[6]);
      display.setCursor(0,44); 
      display.print(TouchTim[7]);display.print(F(" "));
      display.print(TouchTim[8]);display.print(F(" "));
      display.print(TouchTim[9]);
      //  {4,7,5,2,8,3,9,6,1,0}; // La position sur un clavier standard 0 123 456 589    
      display.display();   delay(200);  display.clearDisplay(); }  
  }
  
  display.setTextColor(WHITE, BLACK); // display.clearDisplay(); // 
  if (NTouch == 0 && MOD != 13) {
    if (!NoDisplay) {
      display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI"));
      display.setTextSize(2); display.setCursor(0,24); display.print(F(" No Touch ")); 
      display.setCursor(0,44); display.print(F(" MODE : "));display.print (MOD);  
      display.display();} 
  }  
  else { 
    ComptAutoOff = 0; // Pas d'extinction auto en vu (No AUTO OFF)
  }

  if (ComptAutoOff > ComptAutoOffMAX) {TouchMod[0]=1; TouchMod[7]=1;} // Actionner AutoOFF
    
  if (TouchMod[0] && TouchMod[7]) { // Coupe alim si 4 et 6 (num clavier) sont appuyés en meme temps
    TouchMod[0]=0; TouchMod[7]=0; NTouch=0; OldMOD = MOD; MOD = 3;
    if (!NoDisplay) {
      display.setTextColor(WHITE, BLACK); display.clearDisplay(); 
      display.setTextSize(2); display.setCursor(0,8); display.println(F(" KEY MIDI"));
      display.setTextSize(2); display.setCursor(0,32); display.print(F(" AUTO OFF ")); 
      display.display();display.clearDisplay();}
    delay (3000);digitalWrite(AutoOffPin, LOW); 
    // si alim USB on continu
    digitalWrite(ledVt, LOW); 
    MOD = OldMOD; ComptAutoOff = 0;
  }

  if (TouchMod[3] && TouchMod[4]) { // Affiche la RAM libre
    TouchMod[3]=0; TouchMod[4]=0; NTouch=0;
    if (!NoDisplay) {
      display.setTextColor(WHITE,BLACK); display.clearDisplay(); 
      display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
      display.setTextSize(2); display.setCursor(0,24);  display.print(F(" FREE RAM ")); 
      display.setTextSize(2); display.setCursor(0,44);  display.print(freeMemory() ); 
      display.display();   delay(1000);  display.clearDisplay(); }
    Serial.end(); delay (2000); Serial.begin (31250); // Repasse en mode MIDI
  }
  
  if (TouchMod[1] && TouchMod[6]) { // RESET MIDI (Pas toujours fonctionnel)
    TouchMod[1]=0; TouchMod[6]=0; NTouch=0;
    //Serial.write(B11111111);
    Serial.write(121); // RESET ALL Controleurs ???
    }
  
    
  if (TouchMod[1] && TouchMod[5]) { // Defrag EEPROM
    TouchMod[1]=0; TouchMod[5]=0; NTouch=0;
    if (!NoDisplay) {
      display.setTextColor(WHITE,BLACK); display.clearDisplay(); 
      display.setTextSize(2); display.setCursor(0,8); display.println(F(" KEY MIDI "));
      display.setTextSize(2); display.setCursor(0,32);  display.print(F("  DEFRAG ")); 
      display.display(); display.clearDisplay();} 
    // DEFRAG
    Defrag(); delay(1000); // 1020-AdressEEpromMax bytes libres
    if (!NoDisplay) {
      display.setTextColor(WHITE,BLACK); display.clearDisplay(); 
      display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
      display.setTextSize(2); display.setCursor(0,24);  display.print(F("  LIBRE : ")); 
      display.setTextSize(2); display.setCursor(0,44);  display.print(1020-AdressEEpromMax); 
      display.display(); display.clearDisplay();} 
    delay(2000);  
  }
  
  if (TouchMod[5] && TouchMod[6]) { // Affiche sur la liaison serie l'EEPROM
    Serial.end(); delay (2000);
    Serial.begin (9600);  Serial.println("Hello you"); // Pour les tests et le relevé de l'EEPROM
  
    TouchMod[5]=0; TouchMod[6]=0; NTouch=0;
    Serial.println("");Serial.println(F("EEPROM : "));
    for (i =0; i<18 ;i++) {
      Serial.print(F("Pad "));
      if (i<9) {Serial.print(TouchCod[i]);} else {Serial.print(TouchCod[i-9]);Serial.print(F("'"));}
      FatAdress = 1 + 3 * i; // Pour la bank 1
      AdressEEprom = (EEPROM.read(FatAdress) << 8) + EEPROM.read(FatAdress+1);
      TaillEEprom = EEPROM.read(FatAdress+2);
      Serial.print(F(" : Adrs "));Serial.print(AdressEEprom);
      Serial.print(F(" : Tail "));Serial.print(TaillEEprom);
      Serial.print(F(" :"));
        if (TaillEEprom != 0) {
          //while (TaillEEprom > 0) {Serial.write(EEPROM.read(AdressEEprom++));TaillEEprom--; } // Bien pour du texte d'essai
          //while (TaillEEprom > 0) {Serial.print(EEPROM.read(AdressEEprom++));Serial.print(":");TaillEEprom--; }
          while (TaillEEprom > 0) {Serial.print(EEPROM.read(AdressEEprom++),HEX);Serial.print(":");TaillEEprom--; }
        }
        else { Serial.print(F(" Vide !"));} 
        Serial.println("");
        }  
  }

  if (TouchMod[8] && TouchMod[1]) { // Format EEPROM
    TouchMod[8]=0; TouchMod[1]=0; NTouch=0;
    if (!NoDisplay) {
      display.setTextColor(WHITE,BLACK); display.clearDisplay(); 
      display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
      display.setTextSize(2); display.setCursor(0,32);  display.print(F("FORMAT ???"));
      display.display(); display.clearDisplay();}
    delay(2000);OldMOD = MOD; MOD = 9; NTouch = 0;  
    while (ComptAutoOff < 100 && (NTouch==0)) {GestionTouch();}
    if (TouchMod[9] && (ComptAutoOff < 100) ) { // Le formatage est validé
      if (!NoDisplay) {    
        display.setTextColor(WHITE,BLACK); display.clearDisplay(); 
        display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
        display.setTextSize(2); display.setCursor(0,32);  display.print(F("FORMAT ..."));
        display.display(); display.clearDisplay();}
      delay(2000);  
      for (i=0; i<18;i++) {
        AdressEEprom = 55 + i ; // Les adresses après formatage sont empiler et de 1 byte
        EEPROM.write(1+3*i, AdressEEprom >> 8);// Position byte haut 
        EEPROM.write(2+3*i, AdressEEprom & B11111111);// Position byte bas
        EEPROM.write(3+3*i, 0);// Longueur = 0
      }
    }
    MOD = OldMOD; NTouch = 0; TouchMod[9] =0;
  }
  
  if (!TouchMod[9] && (NTouch>1) ) { // Jouer note de musique pour tester OUT liaison MIDI si deux touches appuyées
    MidiNote(0x90, note, 0x45); delay(500); MidiNote(0x90, note, 0x00); }
    digitalWrite(ledVt, LOW);
    
  if (!TouchMod[9] && (NTouch==1) ) { // Une seule touche du pavé on doit exécuter la séquence MIDI ou une note si vide
    NTouch = 0; // pour éviter de relancer la séquence
    for (i=0; i<10;i++) {
      if (TouchMod[i] != 0) {
        TouchMod[i] = 0; if (MOD == 1) {i = i + 9;}
        while(Serial.available () > 0){Serial.read();}
        FatAdress = 1 + 3 * i; // Pour la bank 1
        AdressEEprom = (EEPROM.read(FatAdress) << 8) + EEPROM.read(FatAdress+1);
        TaillEEprom = EEPROM.read(FatAdress+2);
        if (TaillEEprom != 0) {
          while (TaillEEprom > 0) {
            Serial.write(EEPROM.read(AdressEEprom++));TaillEEprom--;
          }
          if (!NoDisplay) {    
            display.setTextColor(WHITE, BLACK); display.clearDisplay(); 
            display.setTextSize(2); display.setCursor(0,8); display.println(F(" KEY MIDI "));
            display.setTextSize(2); display.setCursor(0,32);  display.print(F("RECORD : "));display.print(TouchCod[i % 9]); 
            display.display(); delay (500);  display.clearDisplay();  }
          delay (500);  
        }  
        else {
          note = 0x50; MidiNote(0x90, note, 0x45); delay(500); MidiNote(0x90, note, 0x00); // Jouer une note
          if (!NoDisplay) {    
            display.setTextColor(WHITE, BLACK); display.clearDisplay(); 
            display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
            display.setTextSize(2); display.setCursor(0,24);  display.print(F("NO RECORD "));
            display.setTextSize(2); display.setCursor(0,44);  display.print(F(" ON PAD ")); display.print(TouchCod[i % 9]);
            if (i>8) { display.print("'");}
            display.display();delay (2000); display.clearDisplay();}
        }
      i = 10; // pour terminer la recherche plus rapidement  
      }
    }
    digitalWrite(ledVt, LOW);
  }
  
  if (TouchMod[9] && (NTouch==1) ) { // On a appuyé sur le bouton MOD
    // Est-ce un appui court ou un appui long ???
    NTouch = 0;delay (500);
    for (i=0; i<20;i++) { // Faire 20 tests de 100ms soit 2s
      TouchTim[9] = 0; TouchMod[9]=0;
      pinMode(TouchPin[9], OUTPUT); digitalWrite(TouchPin[9], HIGH); delay (10); pinMode(TouchPin[9], INPUT);
      while (TouchTim[9]++ < 250 && digitalRead(TouchPin[9])) {}
      if (TouchTim[9]>(TouchBas[9]+Seuil)) {NTouch++;}
      else if (TouchTim[9] <(TouchBas[9]-Seuil)) {NTouch++;}
      else {NTouch--;}
      if ((NTouch == 0) || (NTouch>100)) {i = 20;NTouch = 0;}
      delay (100);
    }
    TouchTim[9] = 0; TouchMod[9]=0;  
    if (NTouch == 0) { // C'est un appui court on bascule de MOD
      if (MOD == 0) {MOD = 1;} else {MOD = 0;}
      digitalWrite(ledVt, LOW);delay (500);
    }
    else { // C'est un appui Long alors il faut attendre et lire le clavier et le buffer MIDI
      ProGramKey ();
    }  
  }
  
  digitalWrite(ledVt, LOW);
  delay(200);
}
//**********************************************************************************
void GestionTouch(void){
  NTouch =0;
  for (i=0; i<10;i++) {
    TouchTim[i] =0;TouchMod[i]=0;
    pinMode(TouchPin[i], OUTPUT); digitalWrite(TouchPin[i], HIGH); delay (10); pinMode(TouchPin[i], INPUT);
    while (TouchTim[i]++ < 250 && digitalRead(TouchPin[i])) {}
    if (TouchTim[i]>250) {TouchTim[i]=0;} // Cas d'un problème de lecture
    if (TouchTim[i]>(TouchBas[i] + Seuil)) {TouchMod[i]=1;NTouch++;} // Touche active enregistrée
    
    //Serial.print(" / "); Serial.print(TouchTim[i]); // For TESTs
  }
  //Serial.print(" / Soit "); Serial.println(NTouch); // For TESTs
}  
//**********************************************************************************
void MidiNote(int cmd, int pitch, int velocity) {
  Serial.write(cmd);
  Serial.write(pitch);
  Serial.write(velocity);
}
//**********************************************************************************
void Defrag() {
          AdressEEpromMin = 55;
          for (i=0; i < 18; i++) { // Pour les 18 memoires
            // Chercher l'adresse la plus basse (mais plus haute que le mini) et la déplacer vers le bas si possible ...
            AdressEEpromMax = 2000;
            for (j=0; j<18;j++) { // Trouver celle dont l'adresse est la plus basse mais supérieure au minimin
              FatAdress = 1 + 3 * j; 
              AdressEEprom = (EEPROM.read(FatAdress) << 8) + EEPROM.read(FatAdress+1);
              if ((AdressEEprom < AdressEEpromMax) && (AdressEEprom > AdressEEpromMin)) {AdressEEpromMax = AdressEEprom; jMax = j;}
            }
            AdressEEpromMin++;
            if (AdressEEpromMax != 2000) { // Si on la trouve, on décale vers le bas et on met à jour la Fat
              // Il faut décaler vers la nouvelle AdressEEpromMin
              FatAdress = 1 + 3 * jMax; TaillEEprom = EEPROM.read(FatAdress+2); 
              AdressEEprom = (EEPROM.read(FatAdress) << 8) + EEPROM.read(FatAdress+1);
              EEPROM.write(FatAdress, AdressEEpromMin >> 8); // Position byte haut 
              EEPROM.write(FatAdress + 1, AdressEEpromMin & B11111111); // Position byte bas
              EEPROM.write(FatAdress + 2 , TaillEEprom);// Taille de l'enregistrement  
              while (TaillEEprom > 0) {EEPROM.write(AdressEEpromMin++,EEPROM.read(AdressEEprom++));TaillEEprom--;}
            }
          AdressEEpromMax = AdressEEpromMin; // A la fin il faux renvoyer la premiere adresse libre
          }
}  
//********************************************************************************** 
void ProGramKey (){
      // Vider le buffer MIDI hard et soft
      CmptMidi = 0; // byte BufferMidi [300];
      while(Serial.available () > 0){Serial.read();}
      delay (2000);      
      Timer1.detachInterrupt();
      digitalWrite(ledRg, HIGH);      digitalWrite(ledVt, HIGH);
      OldMOD = MOD; MOD = 2; NTouch = 0;
      if (!NoDisplay) {    
        display.setTextColor(WHITE, BLACK); display.clearDisplay(); 
        display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
        display.setTextSize(2); display.setCursor(0,24);  display.print(F("  RECORD  "));
        display.setTextSize(2); display.setCursor(0,44);  display.print(F("   MIDI   "));  
        display.display(); }
      while(Serial.available () > 0){Serial.read();}
      delay (2000);      
      // Vider le buffer MIDI hard et soft
      CmptMidi = 0; // byte BufferMidi [300];
      while(Serial.available () > 0){Serial.read();}
      // Lire buffer hard et wait clavier
      GestionTouch(); 
      while ((NTouch == 0) && (CmptMidi < BufferMidiMax)) {
        GestionTouch();
        // Lecture éventuelle de la réception midi  
        while ((Serial.available() > 0) && (CmptMidi < BufferMidiMax)) {
          midiByte = Serial.read(); // ATTENTION F0 début et F7 fin SysEx messages
          if (midiByte == 254) {midiByte = 0;} // Sauter les mauvaises infos
          else if ((midiByte & B11111000) == B11111000) {midiByte = 0;} // Sauter les 'Sys real time'
          else {
            BufferMidi [CmptMidi++] = midiByte;
          }  
        }
        //digitalWrite(ledVt, LOW);delay (10); digitalWrite(ledVt, HIGH);
        if (1 == 0) {    // Sinon le buffer de 63 octets n'est pas déchargé assez vite
          display.setCursor(0,24);  display.print(F("REC :     "));
          display.setCursor(72,24);  display.print(midiByte);
          display.setCursor(0,44);  display.print(F("Num :     "));  
          display.setCursor(72,44);  display.print(CmptMidi); display.display();}
      } // Fin d'attente MIDI
      if (!NoDisplay) {display.setTextColor(WHITE, BLACK); display.clearDisplay();} 
      if ((CmptMidi >= BufferMidiMax) || TouchMod[9] || (CmptMidi == 0)) { // Annuler l'enregistrement 
        MOD = 6;
        if (!NoDisplay) {    
          display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
          display.setTextSize(2); display.setCursor(0,24);  display.print(F("NO RECORD "));
          display.setTextSize(2); display.setCursor(0,44);  display.print(CmptMidi); display.print(F("  Bytes"));  
          display.display(); display.clearDisplay();}  
        delay (3000);  
      }
      else { // Stoker buffer dans EEprom si possible, sinon tenter une défrag
        MOD = 7; 
        if (!NoDisplay) {    
          display.setTextSize(2); display.setCursor(0,4); display.println(F(" KEY MIDI "));
          display.setTextSize(2); display.setCursor(0,24);  display.print(F("  RECORD  "));
          display.setTextSize(2); display.setCursor(0,44);  display.print(CmptMidi); display.print(F("  Bytes"));  
          display.display();  display.clearDisplay();}  
        delay (3000);
        //chercher la fin des enregistrements
        AdressEEpromMax = 0;
        for (i=0; i<18;i++) {
          FatAdress = 1 + 3 * i; 
          AdressEEprom = (EEPROM.read(FatAdress) << 8) + EEPROM.read(FatAdress+1);
          if (AdressEEprom > AdressEEpromMax) {AdressEEpromMax = AdressEEprom; iMax = i;}
        }
        AdressEEpromMax = AdressEEpromMax + EEPROM.read(3 + 3 * iMax); //AdressEEpromMax contient la premiere Adresse libre
        // On commence par chercher le TouchMod[i] à 1 qu'il faut modifier
        Pad = 0; while (TouchMod[Pad] == 0 && Pad<9) {Pad++;}
        // On regarde si ça passe pas et on fait un defrag
        if ((AdressEEpromMax + CmptMidi) > 1020) { // Alors il faut faire une défrag
          Defrag();
        }
        if ((AdressEEpromMax + CmptMidi) > 1020) { // Alors la mise en mémoire est impossible !!!
          Pad = 10; // Pour signifier le problème        
        }
        if (Pad<9) {
          if (OldMOD == 1) {Pad = Pad + 9;}
          // On modifie la Fat en EEprom 
          EEPROM.write(1+3*Pad, AdressEEpromMax >> 8); // Position byte haut 
          EEPROM.write(2+3*Pad, AdressEEpromMax & B11111111); // Position byte bas
          EEPROM.write(3+3*Pad, CmptMidi);// Taille de l'enregistrement  
          // On ecrit maintenant le buffer occupé sur l'EEprom
          for (i=0; i < CmptMidi; i++) {EEPROM.write(AdressEEpromMax + i, BufferMidi [i]);};
        }
        else {
          // La défrag n'a pas suffit !!! 
        }  
      }  
    Timer1.attachInterrupt(GestionLeds); // Run every 0.05 seconds 
    MOD = OldMOD; digitalWrite(ledVt, LOW); 
}    
//********************************************************************************** 

 
